﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

// 본예제의 정상 동작을 위해서는 FTP Server를 설정하여 사용해야 합니다. 
// 자세한 사항은 자료실 → SmartXFramework 관련 → Reference Guide → 
// Part 4. 사용자 편의 컴포넌트 → 10. SmartFTP 항목을 참고하시기 바랍니다.
namespace TechNote18_RemoteUpdate_CS
{
    public partial class Form1 : Form
    {
        // strFTPPath는 서버측 파일 경로, strLocalPath는 클라이언트측 파일 경로
        private string[] strFTPPath;
        private string[] strLocalPath;
        private int i = 0;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            smartForm1.MainForm = this;

            // FTP Server측 File경로
            strFTPPath = new string[3];
            // Client측 File경로
            strLocalPath = new string[3];

            // 다운로드 할 파일들의 전체경로를 배열에 저장
            strFTPPath[0] = "UpdateFiles//RemoteUpdate.exe";                // 서버에서 업데이트 할 파일의 경로
            strLocalPath[0] = "SD Card\\UpdateFiles\\RemoteUpdate.exe";     // 로컬에서 파일을 다운로드 받을 경로

            strFTPPath[1] = "UpdateFiles//SmartXNCommon.dll";
            strLocalPath[1] = "SD Card\\UpdateFiles\\SmartXNCommon.dll";

            strFTPPath[2] = "UpdateFiles//SmartXNFDCommon.dll";
            strLocalPath[2] = "SD Card\\UpdateFiles\\SmartXNFDCommon.dll";

            // 리스트 항목 모두 삭제
            lsbFileList.Items.Clear();

            // 다운로드 파일 리스트 표시
            for (int k = 0; k < 3; k++)
            {
                lsbFileList.AddItem(strFTPPath[k]);
            }

            // 파일의 개수 만큼 다운로드 확인하는 타이머
            smartTimer1.Interval = 1000;
            // 각 파일의 다운로드 상태가 100%일 때까지 확인하는 타이머
            smartTimer2.Interval = 100;
            // 전체 파일의 다운로드 완료 시 업데이트 처리를 하는 타이머
            smartTimer3.Interval = 1000;

            // FTP 서버로부터 다운로드 받은 파일의 업데이트 경로를 설정
            smartUpdate1.FromFilePath = "SD Card\\UpdateFiles";
            smartUpdate1.ToFilePath = "Flash Disk\\Run";
            smartUpdate1.Restart = SmartX.SmartUpdate.RESTARTFLAG.AUTORUNEXECUTE;
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // 연결 해제 버튼
            smartFTP1.DisConnect();
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            // 아래 내용은 FTP 서버의 연결 설정 정보를 설정 합니다.
            // 테스트하고자 하시는 FTP 서버 정보로 수정 하세요.
            smartFTP1.ServerFTPAddress = "192.168.1.41";
            smartFTP1.PortNo = 21;
            smartFTP1.PassiveMode = true;
            smartFTP1.UserID = "hns";
            smartFTP1.Password = "";

            // FTP 연결 요청
            if (smartFTP1.Connect() == true)
            {
                labStatus.Text = "연결 성공";
            }
            else
            {
                labStatus.Text = "연결 실패";
            }
        }

        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            smartFTP1.DisConnect();
        }

        // 파일 다운로드 버튼의 클릭 이벤트
        private void btnDown_Click(object sender, EventArgs e)
        {
            // 업데이트 파일을 다운로드 할 디렉터리가 존재하지 않으면 생성
            DirectoryInfo di = new DirectoryInfo("SD Card\\UpdateFiles");
            if (di.Exists == false)
            {
                di.Create();
            }
            
            // 리스트 항목 모두 삭제
            lsbFileList.Items.Clear();

            smartTimer1.Start();
        }

        // 파일 업데이트 버튼의 클릭 이벤트
        private void btnUpdateStart_Click(object sender, EventArgs e)
        {
            smartTimer3.Start();
        }

        // 종료 버튼의 클릭 이벤트
        private void btnClose_Click(object sender, EventArgs e)
        {
            // 연결 해제 버튼
            smartFTP1.DisConnect();

            // 프로그램 종료
            smartForm1.Close();
        }

        private void smartTimer1_Tick(object sender, EventArgs e)
        {
            smartTimer1.Stop();

            // 파일의 개수만큼 동작
            if (i < 3)
            {
                // 파일 다운로드 시작
                smartFTP1.FileDownload(strFTPPath[i], strLocalPath[i]);
                // 다운로드 상태 100% 체크 확인 타이머 시작
                smartTimer2.Start();
            }
            else
            {
                // 전체 파일의 다운로드 완료
                labStatus.Text = "Finish";
                // 파일 다운로드 종료
                smartTimer1.Stop();
                // 전체 파일의 다운로드 완료 시 SmartUpdate로 파일 업데이트 시작
                smartTimer3.Start();
            }
        }

        private void smartTimer2_Tick(object sender, EventArgs e)
        {
            // Percentage가 100%이면 해당 파일의 다운로드가 완료된 상태
            if (smartFTP1.Percentage == 100)
            {
                // ListBox에 [완료] 메시지를 출력
                lsbFileList.AddItem("[완료]" + strFTPPath[i]);
                // 다음 파일 다운로드를 위해 i값 1증가
                ++i;
                // 다음 파일 다운로드를 위해 smartTimer2 정지
                smartTimer2.Stop();
                // 다음 파일 다운로드를 위해 smartTimer1 시작
                smartTimer1.Start();
            }
            // -1이면 다운로드 실패
            else if (smartFTP1.Percentage == -1)
            {
                // ListBox에 [실패] 메시지를 출력
                lsbFileList.AddItem("[실패]" + strFTPPath[i]);
                ++i;
                smartTimer2.Stop();
                smartTimer1.Start();
            }
            else
            {
                // 현재 파일의 다운로드 %를 표시
                labStatus.Text = smartFTP1.Percentage.ToString() + "%";
            }
        }

        // SmartUpdate를 위한 SmartTimer Tick Event
        private void smartTimer3_Tick(object sender, EventArgs e)
        {
            smartTimer3.Stop();

            // 업데이트 시작의 지연 시간을 설정
            smartUpdate1.UpdateStartInterval = 3000;

            if (smartUpdate1.UpdateCheck() == true)
            {
                DialogResult dResult = SmartX.SmartMessageBox.Show("업데이트를 시작 하시겠습니까?", "업데이트 시작", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (dResult == DialogResult.Yes)
                {
                    smartUpdate1.Start();
                    smartForm1.Close();
                }
            }
            else
            {
                SmartX.SmartMessageBox.Show("현재 파일이 최신 버전이거나 업데이트 파일이 존재하지 않습니다.");
            }
        }
    }
}